/*!
    \file    main.h
    \brief   the header file of main

    \version 2024-12-20, V1.2.0, firmware for GD32F5xx
*/

/*
    Copyright (c) 2024, GigaDevice Semiconductor Inc.

    Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this 
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice, 
       this list of conditions and the following disclaimer in the documentation 
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors 
       may be used to endorse or promote products derived from this software without 
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
*/

#include "gd32f5xx.h"
#include "stdio.h"

#ifndef MAIN_H
#define MAIN_H

__attribute__((aligned(32)))
const uint8_t message_input[] = {
    0x43, 0x48, 0x4E, 0x20, 0x47, 0x69, 0x67, 0x61, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x53,
    0x65, 0x6D, 0x69, 0x63, 0x6F, 0x6E, 0x64, 0x75, 0x63, 0x74, 0x6F, 0x72, 0x20, 0x49, 0x6E, 0x63,
    0x43, 0x48, 0x4E, 0x20, 0x47, 0x69, 0x67, 0x61, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x53,
    0x65, 0x6D, 0x69, 0x63, 0x6F, 0x6E, 0x64, 0x75, 0x63, 0x74, 0x6F, 0x72, 0x20, 0x49, 0x6E, 0x63,
    0x43, 0x48, 0x4E, 0x20, 0x47, 0x69, 0x67, 0x61, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x53,
    0x65, 0x6D, 0x69, 0x63, 0x6F, 0x6E, 0x64, 0x75, 0x63, 0x74, 0x6F, 0x72, 0x20, 0x49, 0x6E, 0x63,
    0x43, 0x48, 0x4E, 0x20, 0x47, 0x69, 0x67, 0x61, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x53,
    0x65, 0x6D, 0x69, 0x63, 0x6F, 0x6E, 0x64, 0x75, 0x63, 0x74, 0x6F, 0x72, 0x20, 0x49, 0x6E, 0x63,
    0x43, 0x48, 0x4E, 0x20, 0x47, 0x69, 0x67, 0x61, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x53,
    0x65, 0x6D, 0x69, 0x63, 0x6F, 0x6E, 0x64, 0x75, 0x63, 0x74, 0x6F, 0x72, 0x20, 0x49, 0x6E, 0x63,
    0x43, 0x48, 0x4E, 0x20, 0x47, 0x69, 0x67, 0x61, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x53,
    0x65, 0x6D, 0x69, 0x63, 0x6F, 0x6E, 0x64, 0x75, 0x63, 0x74, 0x6F, 0x72, 0x20, 0x49, 0x6E, 0x63,
    0x43, 0x48, 0x4E, 0x20, 0x47, 0x69, 0x67, 0x61, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x53,
    0x65, 0x6D, 0x69, 0x63, 0x6F, 0x6E, 0x64, 0x75, 0x63, 0x74, 0x6F, 0x72, 0x20, 0x49, 0x6E, 0x63,
    0x43, 0x48, 0x4E, 0x20, 0x47, 0x69, 0x67, 0x61, 0x44, 0x65, 0x76, 0x69, 0x63, 0x65, 0x20, 0x53,
    0x65, 0x6D, 0x69, 0x63, 0x6F, 0x6E, 0x64, 0x75, 0x63, 0x74, 0x6F, 0x72, 0x20, 0x49, 0x6E, 0x63
};

__attribute__((aligned(32)))
const uint8_t expected_data_sha224[] = {
    0xD3, 0x91, 0x8F, 0x90,
    0x2E, 0xB5, 0xDC, 0x54,
    0x2D, 0xEB, 0x4A, 0x7D,
    0x48, 0x62, 0x27, 0x37,
    0x7F, 0x06, 0x24, 0x6E,
    0xBF, 0x05, 0x99, 0xF9,
    0x53, 0xF9, 0x38, 0xF7
};

__attribute__((aligned(32)))
const uint8_t expected_data_sha256[] = {
    0xC6, 0x43, 0xD7, 0x88,
    0xA2, 0x91, 0x05, 0x61,
    0xEF, 0xFC, 0xFA, 0xD8,
    0x5D, 0x95, 0xCC, 0x43,
    0x7B, 0x9E, 0x1E, 0x18,
    0x51, 0x97, 0x3A, 0x41,
    0x60, 0xDB, 0xE5, 0xFC,
    0xA1, 0x26, 0x42, 0xB5
};

#endif /* MAIN_H */
